@@ -3,20 +3,23 @@
 Author: David J. Morfe
 Application Name: GeoLiberator
 Functionality Purpose: Instill data quality upon address data
-Version: Beta 0.2.9
+Version: Beta
 '''
-#8/12/19
+#8/6/20
 
 import re
 import sys
 import time
+import pandas as pd
 
 #Account for post cardinal direction
 #Account for '&' and 'STS' and multiple street types
-#Option to append borough, state, zip, based on argument
 #Create custom address formatter
 
-reason = ["Invalid parse argument given"]
+#Function to parse city
+#Allow street types to be appended to library or option to use one's own library
+
+reason = ["Invalid parse argument given", "File type not supported", "Use 'address_field' argument for csv & excel files"]
 
 class AddressError(BaseException):
     pass
@@ -47,7 +50,7 @@ class GeoLiberator:
     Sample Code:
         from geoliberator import *
         GL_Object = GeoLiberator("123 Sample Street, New York 12345")
-        GL_Object.getAddress(log="output_log.txt", mode=True) #This code should both print and create a log file
+        GL_Object.getAddress(log="output_log.txt") #This code appends parsed address to a log file (useful in loops)
     '''
     
     def __init__(self, addr):
@@ -108,6 +111,11 @@ class GeoLiberator:
         for typ in self.streetTypes:
             hold += re.sub(r"[\[\]' ]", '', str(self.streetTypes[typ])) + ','
         self.streetTypesAll = list(hold.strip(',').split(','))
+    def __none_other(self, *args):
+        for val in args:
+            if val == "OTHER":
+                return False
+        return True
 
     def _get_compass(self, direc):
         if 'N' == direc or "NO" == direc:
@@ -135,24 +143,6 @@ class GeoLiberator:
             nn = num + 'th'
         return nn
 
-    def _get_state(self, log=''):
-        get = (self.addr).upper(); full_state = '' #Uppercase and create new address to return
-        get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
-        for key, val in self.states.items():
-            if re.search(fr"\b{val}\b", get):
-                full_state = key
-        if full_state == '':
-            full_state = "OTHER"
-
-        if log != '': #Write to new or specfied file
-            fileName = re.sub(r"\..+", '', log)
-            if fileName.isdigit() or re.search(r'[\/:*?"<>|]', fileName):
-                fileName = "newly_parsed_states"
-            nf = open(f"{fileName}.txt", 'a')
-            nf.write(full_state + '\n')
-            nf.close()
-        return str(full_state).upper()
-
     def _search_cycle(self, g, sF):
         new_find = ''
         if sF == False:
@@ -198,8 +188,43 @@ class GeoLiberator:
             new_find = "OTHER"
         return new_find
 
+    def get_zip(self, log=''):
+        get = (self.addr).upper(); full_zip = '' #Uppercase and create get zipcode to return
+        get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+        if re.search(r"\b\d{5}\b", get):
+            full_zip = re.search(r"\b\d{5}\b", get).group()
+        else:
+            full_zip = "OTHER"
+
+        if log != '': #Write to new or specfied file
+            fileName = re.sub(r"\..+", '', log)
+            if fileName.isdigit() or re.search(r'[\/:*?"<>|]', fileName):
+                fileName = "newly_parsed_zipcodes"
+            nf = open(f"{fileName}.txt", 'a')
+            nf.write(full_zip + '\n')
+            nf.close()
+        return full_zip
+
+    def get_state(self, log=''):
+        get = (self.addr).upper(); full_state = '' #Uppercase and find full state to return
+        get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+        for key, val in self.states.items():
+            if re.search(fr"\b{val}\b", get):
+                full_state = key
+        if full_state == '':
+            full_state = "OTHER"
+
+        if log != '': #Write to new or specfied file
+            fileName = re.sub(r"\..+", '', log)
+            if fileName.isdigit() or re.search(r'[\/:*?"<>|]', fileName):
+                fileName = "newly_parsed_states"
+            nf = open(f"{fileName}.txt", 'a')
+            nf.write(full_state + '\n')
+            nf.close()
+        return str(full_state).upper()
+
     def getAddressNum(self, log=''):
-        get = (self.addr).upper(); new_addr_num = '' #Uppercase and create new address to return
+        get = (self.addr).upper(); new_addr_num = '' #Uppercase and create get house number to return
         if not get[0].isdigit():
             for key, val in self.states.items():
                 if re.search(fr"^{val}\b", get):
@@ -262,7 +287,7 @@ class GeoLiberator:
         return str(new_addr_num)
 
     def getStreet(self, log=''):
-        get = (self.addr).upper(); new_street = ''; saintFlag = False #Uppercase and create new address to return
+        get = (self.addr).upper(); new_street = ''; saintFlag = False #Uppercase and get street name to return
         if '/' in get:
             for key, val in self.states.items():
                 if re.search(fr"\b{val}\b", get):
@@ -302,7 +327,7 @@ class GeoLiberator:
         get = (self.addr).upper(); new_addr = '' #Uppercase and create new address to return
         gS = GeoLiberator(get).getStreet()
         gAN = GeoLiberator(get).getAddressNum()
-        if gAN != "OTHER" and gS != "OTHER":
+        if self.__none_other(gS, gAN):
             new_addr = gAN + ' ' + gS
         else:
             new_addr = "OTHER"
@@ -316,66 +341,105 @@ class GeoLiberator:
             nf.close()
         return new_addr
 
-#Acount the lines in a file
+    def full_address(self, log=''):
+        get = (self.addr).upper(); full_addr = '' #Uppercase and create 'full' address to return
+        get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+        gA = GeoLiberator(get).getAddress()
+        gST = GeoLiberator(get).get_state()
+        gZP = GeoLiberator(get).get_zip()
+        if self.__none_other(gA, gST, gZP):
+            full_addr = gA + ', ' + gST + ' ' + gZP
+        else:
+            full_addr = "OTHER"
+
+        if log != '': #Write to new or specfied file
+            fileName = re.sub(r"\..+", '', log)
+            if fileName.isdigit() or re.search(r'[\/:*?"<>|]', fileName):
+                fileName = "newly_parsed_addresses"
+            nf = open(f"{fileName}.txt", 'a')
+            nf.write(full_addr + '\n')
+            nf.close()
+        return full_addr
+
+#Count the lines in a file
 def file_len(file_name):
     with open(file_name) as f:
         for i, L in enumerate(f):
             pass
     return i + 1
 
-#Takes text file as input and switch argument to determine which address property to be standardized
-def autoGeoLiberate(file_path, parse="address", write=''):
-    mode = True
+#Takes text file as input and parse argument to determine which address property to be standardized
+def autoGeoLiberate(file_path: str, address_field='', parse="address", write=''):
+    mode = True; lines = ''
     if not re.search(r"address|number|street", parse):
         raise ArgumentError(reason[0])
     if write != '':
         mode = False
-    with open(file_path) as f:
-        lines = f.readlines()
-        if len(sys.argv) == 2 and write != '':
-            if sys.argv[1] == '--status' or sys.argv[1] == "-S":
-                FL = file_len(file_path)
-                barIncr = int(FL * .025); barNum = 0; dashNum = 40; c = 0; lc = 0
-                print('|' + ('-' * 40) + '|' + " [0.00%]", end=''); sys.stdout.flush()
-                for line in lines:
-                    perc = (lc/FL)
-                    bar = '\r|' + ('█' * barNum) + ('-' * dashNum) + '|' + " [{:>.2%}]".format(perc)
-                    print(bar, end=''); sys.stdout.flush()
-                    c += 1; lc += 1
-                    if c == barIncr:
-                        if barNum < 39:
-                            c = 0; barNum += 1; dashNum -= 1
-                            print(bar, end=''); sys.stdout.flush()
-                    elif lc == FL:
-                        print('\r|' + ('█' * 40) + '|' + " [100%]  "); sys.stdout.flush()
-                    adr = GeoLiberator(str(line))
-                    if parse.lower() == "address":
-                        adr.getAddress(log=write)
-                    elif parse.lower() == "number":
-                        adr.getAddressNum(log=write)
-                    elif parse.lower() == "street":
-                        adr.getStreet(log=write)
-                    elif parse.lower() == "state":
-                        adr._get_state(log=write)
+    if address_field == '':
+        if re.search(r".xlsx?$", file_path) or re.search(r".csv$", file_path):
+            raise ArgumentError(reason[2])
+        with open(file_path) as f:
+            lines = f.readlines()
+    else:
+        if re.search(r".xlsx?$", file_path):
+            df = pd.read_excel(file_path, usecols=[str(address_field)])
+        elif re.search(r".csv$", file_path):
+            df = pd.read_csv(file_path, usecols=[str(address_field)])
         else:
-            if mode == False:
-                print("Running...")
+            raise ArgumentError(reason[1])
+        lines = df[str(address_field)]
+    if len(sys.argv) == 2 and write != '':
+        if sys.argv[1] == '--status' or sys.argv[1] == "-S":
+            FL = file_len(file_path)
+            barIncr = int(FL * .025); barNum = 0; dashNum = 40; c = 0; lc = 0
+            print('|' + ('-' * 40) + '|' + " [0.00%]", end=''); sys.stdout.flush()
             for line in lines:
+                perc = (lc/FL)
+                bar = '\r|' + ('█' * barNum) + ('-' * dashNum) + '|' + " [{:>.2%}]".format(perc)
+                print(bar, end=''); sys.stdout.flush()
+                c += 1; lc += 1
+                if c == barIncr:
+                    if barNum < 39:
+                        c = 0; barNum += 1; dashNum -= 1
+                        print(bar, end=''); sys.stdout.flush()
+                elif lc == FL:
+                    print('\r|' + ('█' * 40) + '|' + " [100%]  "); sys.stdout.flush()
                 adr = GeoLiberator(str(line))
                 if parse.lower() == "address":
-                    out = adr.getAddress(log=write)
+                    adr.getAddress(log=write)
                 elif parse.lower() == "number":
-                    out = adr.getAddressNum(log=write)
+                    adr.getAddressNum(log=write)
                 elif parse.lower() == "street":
-                    out = adr.getStreet(log=write)
+                    adr.getStreet(log=write)
                 elif parse.lower() == "state":
-                    out = adr._get_state(log=write)
-                if mode == True:
-                    print(out)
-            print("Done!")
+                    adr.get_state(log=write)
+                elif parse.lower() == "zipcode":
+                    out = adr.get_zip(log=write)
+                elif parse.lower() == "full":
+                    out = adr.full_address(log=write)
+    else:
+        if mode == False:
+            print("Running...")
+        for line in lines:
+            adr = GeoLiberator(str(line))
+            if parse.lower() == "address":
+                out = adr.getAddress(log=write)
+            elif parse.lower() == "number":
+                out = adr.getAddressNum(log=write)
+            elif parse.lower() == "street":
+                out = adr.getStreet(log=write)
+            elif parse.lower() == "state":
+                out = adr.get_state(log=write)
+            elif parse.lower() == "zipcode":
+                out = adr.get_zip(log=write)
+            elif parse.lower() == "full":
+                out = adr.full_address(log=write)
+            if mode == True:
+                print(out)
+        print("Done!")
 
-#Takes address as input and switch argument to determine which address property to be standardized
-def geoLiberate(addr, parse="address"):
+#Takes address as input and parse argument to determine which address property to be standardized
+def geoLiberate(addr: str, parse="address"):
     adr = GeoLiberator(str(addr))
     try:
         if parse.lower() == "address":
@@ -385,7 +449,31 @@ def geoLiberate(addr, parse="address"):
         elif parse.lower() == "street":
             out = adr.getStreet()
         elif parse.lower() == "state":
-            out = adr._get_state()
+            out = adr.get_state()
+        elif parse.lower() == "zipcode":
+            out = adr.get_zip()
+        elif parse.lower() == "full":
+            out = adr.full_address()
         print(out)
     except (AttributeError, UnboundLocalError):
         raise ArgumentError(reason[0])
+
+#Returns standardized address based on parse argument
+def parse_address(addr: str, parse="address") -> str:
+    adr = GeoLiberator(str(addr))
+    try:
+        if parse.lower() == "address":
+            out = adr.getAddress()
+        elif parse.lower() == "number":
+            out = adr.getAddressNum()
+        elif parse.lower() == "street":
+            out = adr.getStreet()
+        elif parse.lower() == "state":
+            out = adr.get_state()
+        elif parse.lower() == "zipcode":
+            out = adr.get_zip()
+        elif parse.lower() == "full":
+            out = adr.full_address()
+        return str(out)
+    except (AttributeError, UnboundLocalError):
+        raise ArgumentError(reason[0])
